#!/usr/bin/env python3
"""
r2inspect package main entry point
"""

# Import main from cli_main.py (renamed from cli.py to avoid conflict with cli/ package)
from r2inspect.cli_main import main

if __name__ == "__main__":
    main()
